/*
 * Created on Jan 6, 2005
 */
package gov.grants.apply.xpath;

import gov.grants.apply.Log4JInit;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;

import javax.xml.transform.TransformerException;

import org.apache.log4j.Logger;
import org.w3c.dom.Node;

import junit.framework.Assert;
import junit.framework.TestCase;
import junit.textui.TestRunner;

/**
 * A Unit test for the XPathExecutor class.
 * 
 * @author David Wong
 */
public class XPathExecutorTest
    extends TestCase
{
    private final static Logger LOG = Logger.getLogger(XPathExecutorTest.class
        .getName());

    private static Log4JInit init = new Log4JInit(); // load the class so that
    // log4j is initialized,
    // doesn't do anything
    // else.

    private XPathExecutor _xpathExe = null;

    public static void main(String[] args)
    {
        TestRunner.run(XPathExecutorTest.class);
    }

    /**
     * Read in testFile.xml and initialize a XPathExecutor object with it.
     * 
     * @see TestCase#setUp()
     */
    protected void setUp()
        throws Exception
    {
        super.setUp();
        Reader testFile = null;
        StringBuffer stringform = null;
        try
        {
            testFile = new InputStreamReader(new BufferedInputStream(
                XPathExecutorTest.class.getClassLoader().getResourceAsStream(
                    "resources/testFile.xml")));
            stringform = new StringBuffer();
            char[] buffer = new char[1024];
            int charsRead = 0;
            while ((charsRead = testFile.read(buffer, 0, 1024)) > -1)
            {
                stringform.append(buffer, 0, charsRead);
            }
        }
        finally
        {
            try
            {
                testFile.close();
            }
            catch (IOException e)
            {
            }
        }
        _xpathExe = new XPathExecutor(stringform.toString());
    }

    /*
     * @see TestCase#tearDown()
     */
    protected void tearDown()
        throws Exception
    {
        super.tearDown();
    }

    /**
     * Constructor for XPathExecutorTest.
     * 
     * @param methodName
     */
    public XPathExecutorTest(String methodName)
    {
        super(methodName);
    }

    public final void testGetNode()
    {
        Node hashNode = null;
        try
        {
            hashNode = _xpathExe.getNode(GrantApplicationXpath.HASH_XPATH);
        }
        catch (TransformerException e)
        {
            Assert.fail(e.getMessage());
        }
        Assert.assertNotNull("The XPathExecutor did not return a Node.",
            hashNode);
        LOG.debug(hashNode.getNodeName());
        Assert.assertEquals("The XPathExecutor returned the wrong Node.",
            "glob:HashValue", hashNode.getNodeName());
    }
}