/*
 * Created on Jun 15, 2004
 *
 */
package gov.grants.apply.struts.actions;

import gov.grants.apply.WebServices.ApplicantIntegrationServices_V1_0.ApplicantIntegrationSoapBindingStub;
import gov.grants.apply.WebServices.ApplicantIntegrationServices_V1_0.ApplicationInformationType;
import gov.grants.apply.WebServices.ApplicantIntegrationServices_V1_0.GetApplicationListRequestApplicationFilter;
import gov.grants.apply.WebServices.ApplicantIntegrationServices_V1_0.GetApplicationListRequestApplicationFilterFilter;
import gov.grants.apply.WebServices.ApplicantIntegrationServices_V1_0.GetApplicationListResponse;
import gov.grants.apply.soap.util.SoapUtils;
import gov.grants.apply.struts.Constants;
import gov.grants.apply.struts.forms.GetApplicationListForm;
import gov.grants.apply.struts.util.Utils;
import gov.grants.apply.valueObject.ApplicationListInformation;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;
import org.apache.struts.action.Action;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

/**
 * Action class for getting application list.
 * 
 * @author Feng Qian
 * @author Brian Husted
 * @author Mark Sommer
 *
 */

public class GetApplicationListAction extends Action implements Constants {

	static final Logger log = Logger.getLogger(GetApplicationListAction.class.getName());

	/**
	 * Executes an Action.
	 * @param mapping the ActionMapping.
	 * @param aform the ActionForm.
	 * @param request the HttpServletRequest.
	 * @param response the HttpServletResponse.
	 * @return the ActionForward.
	 * @throws Exception.
	 */
	public ActionForward execute(
			ActionMapping mapping, 
			ActionForm form,
			HttpServletRequest request, 
			HttpServletResponse response )
	throws Exception {

		log.debug( "Get handle to ApplicantIntegrationSoapBindingStub" );
		ApplicantIntegrationSoapBindingStub stub = SoapUtils.getApplicantSoapStub();
		
		log.debug( "Get GetApplicationListResponse" );
		GetApplicationListResponse getApplicationListResponse = 
			stub.getApplicationList( createRequest( ( GetApplicationListForm ) form ) );
		log.debug( "Adding application info" );
		request.getSession().setAttribute(
				APPLICATION_INFO, 
				formatReply( getApplicationListResponse ) );

		return mapping.findForward(ACTION_SUCCESS);

	}
	/**
	 * Creates an application list request
	 * 
	 * @param form the GetApplicationListForm
	 * @return the _GetApplicationListRequest
	 */
	protected GetApplicationListRequestApplicationFilter[] createRequest(
			GetApplicationListForm form) {

		List list = new ArrayList();
		GetApplicationListForm appListForm = (GetApplicationListForm) form;
		log.debug("Getting Filter Values");
//		boolean usingGrantTrackingNum = false;
		
		if ( StringUtils.isNotBlank( appListForm.getTrackingNum() ) ) {
			log.debug( "Only adding tracking number filter" );
			GetApplicationListRequestApplicationFilter applicationFilter_trackingNum = 
				new GetApplicationListRequestApplicationFilter();
			applicationFilter_trackingNum.setFilter( GetApplicationListRequestApplicationFilterFilter.GrantsGovTrackingNumber );
			applicationFilter_trackingNum.setFilterValue(appListForm.getTrackingNum());
			log.debug("Adding a tracking number filter of: " + appListForm.getTrackingNum());
			list.add(applicationFilter_trackingNum);
			
		} else {
			log.debug( "No tracking number -- checking/adding other filter(s)" );
			
			if ( StringUtils.isNotBlank( appListForm.getStatus() ) ) {
				GetApplicationListRequestApplicationFilter applicationFilter_status = new GetApplicationListRequestApplicationFilter();
				applicationFilter_status
						.setFilter(GetApplicationListRequestApplicationFilterFilter.Status);
				applicationFilter_status.setFilterValue(appListForm.getStatus());
				log.debug("Adding a Status filter of: " + appListForm.getStatus());
				list.add(applicationFilter_status);
			}
			
			if ( StringUtils.isNotBlank( appListForm.getCFDANum() ) ) {
				GetApplicationListRequestApplicationFilter applicationFilter_CFDANum = new GetApplicationListRequestApplicationFilter();
				applicationFilter_CFDANum
						.setFilter(GetApplicationListRequestApplicationFilterFilter.CFDANumber);
				applicationFilter_CFDANum.setFilterValue(appListForm.getCFDANum());
				log.debug("Adding a CFDANum filter of: "
								+ appListForm.getCFDANum());
				list.add(applicationFilter_CFDANum);
			}
	
			if ( StringUtils.isNotBlank( appListForm.getOpportunityID() ) ) {
				GetApplicationListRequestApplicationFilter applicationFilter_OpportunityID = new GetApplicationListRequestApplicationFilter();
				applicationFilter_OpportunityID
						.setFilter(GetApplicationListRequestApplicationFilterFilter.OpportunityID);
				applicationFilter_OpportunityID.setFilterValue(appListForm
						.getOpportunityID());
				log.debug("Adding a OppId filter of: "
						+ appListForm.getOpportunityID());
				list.add(applicationFilter_OpportunityID);
			}
			
			if ( StringUtils.isNotBlank( appListForm.getSubmissionTitle() ) ) {
				GetApplicationListRequestApplicationFilter applicationFilter_SubmissionTitle = new GetApplicationListRequestApplicationFilter();
				applicationFilter_SubmissionTitle
						.setFilter(GetApplicationListRequestApplicationFilterFilter.SubmissionTitle);
				applicationFilter_SubmissionTitle.setFilterValue(appListForm
						.getSubmissionTitle());
				log.debug("Adding a SubmissionTitle filter of: "
						+ appListForm.getOpportunityID());
				list.add(applicationFilter_SubmissionTitle);
			}
			
		}// if-else
		
		return (GetApplicationListRequestApplicationFilter[]) list.toArray(new GetApplicationListRequestApplicationFilter[0]);
	}

	/**
	 * 
	 * Formats a reply
	 * 
	 * @param the getApplicationListResponse
	 * @return An List of the ApplicationInformationType
	 * @throws Exception
	 */
	protected List formatReply(GetApplicationListResponse getApplicationListResponse)
			throws Exception {
		ArrayList applicationList = new ArrayList();

		ApplicationInformationType applicationInformationType = null;

		if (getApplicationListResponse != null) {
			ApplicationInformationType[] application = getApplicationListResponse
					.getApplicationInformation();

			if (application != null && application.length > 0) {
				log.debug("The server returned " + " " + application.length
						+ " applications");
				for (int i = 0; i < application.length; i++) {
					ApplicationListInformation wrapper = new ApplicationListInformation();
					if (application[i].getCFDANumber() != null) {
						wrapper.setCFDANumber(application[i].getCFDANumber());
					}
					if (application[i].getOpportunityID() != null) {
						wrapper.setOpportunityID(application[i].getOpportunityID());
					}
					if (application[i].getCompetitionID() != null) {
						wrapper.setCompetitionID(application[i].getCompetitionID());
					}
					if (application[i].getGrants_govTrackingNumber() != null) {
						wrapper.setGrantsGovTrackingNumber(application[i].getGrants_govTrackingNumber());
					}
					if (application[i].getGrantsGovApplicationStatus() != null) {
						wrapper.setStatus(application[i].getGrantsGovApplicationStatus().getValue());
					}
					if (application[i].getStatusDateTime() != null) {
						wrapper.setStatusDate(Utils.formatDate(application[i]
								.getStatusDateTime().getTime()));
					}
					if (application[i].getReceivedDateTime() != null) {
						wrapper.setReceivedDateTime(Utils
								.formatDate(application[i]
										.getReceivedDateTime().getTime()));

					}
					if (application[i].getAgencyTrackingNumber() != null) {
						wrapper.setAgencyTrackingNumber(application[i].getAgencyTrackingNumber());
					}
					if (application[i].getSubmissionTitle() != null) {
						wrapper.setSubmissionTitle(application[i].getSubmissionTitle());
					}
					applicationList.add(wrapper);
				}
			}
		}
		return applicationList;
	}

}