package gov.grants.apply.struts.actions;

import gov.grants.apply.WebServices.ApplicantIntegrationServices_V1_0.ApplicantIntegrationSoapBindingStub;
import gov.grants.apply.WebServices.ApplicantIntegrationServices_V1_0.GetOpportunityListRequest;
import gov.grants.apply.WebServices.ApplicantIntegrationServices_V1_0.OpportunityInformationType;
import gov.grants.apply.soap.util.SoapUtils;
import gov.grants.apply.struts.Constants;
import gov.grants.apply.struts.forms.GetOpportunityListForm;
import gov.grants.apply.struts.util.Utils;
import gov.grants.apply.valueObject.GetOpportunityListInformation;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.apache.struts.action.Action;
import org.apache.struts.action.ActionError;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

/**
 * Action class for getting opportunity list.
 * 
 * @author
 */
public class GetOpportunityListAction extends Action implements Constants {
	static final Logger log = Logger.getLogger(GetOpportunityListAction.class.getName());

	/**
	 * Executes an action.
	 * 
	 * @param mapping
	 *            the ActionMapping.
	 * @param aform
	 *            the ActionForm.
	 * @param request
	 *            the HttpServletRequest.
	 * @param response
	 *            the HttpServletResponse.
	 * @return the ActionForward.
	 * @throws Exception.
	 */
	public ActionForward execute(
			ActionMapping mapping, 
			ActionForm form,
			HttpServletRequest request, 
			HttpServletResponse response )
	throws Exception {

		if ( ValidateData(mapping, request, "opportunity_id", "cfda_number", "competition_id") != true) {
			ActionErrors errors = new ActionErrors();
			errors.add(ActionErrors.GLOBAL_ERROR, new ActionError( "errors.entry.required"));
			saveErrors(request, errors);
			log.warn("None of the fields are fields.");

			return mapping.findForward(ACTION_CONTINUE);
		}// if

		//	=============================================================
		//	 Declaration of Variables
		//	=============================================================
		GetOpportunityListForm golForm = (GetOpportunityListForm) form;
		String stringOpportunityID = golForm.getOpportunity_id();
		String stringCompetitionID = golForm.getCompetition_id();
		String stringCFDANumber = golForm.getCfda_number();

		ApplicantIntegrationSoapBindingStub stub = SoapUtils.getApplicantSoapStub();

		GetOpportunityListRequest getOpp_request = new GetOpportunityListRequest();

		//	==============================================================
		//	 Conversion of Values from the Bean
		// to GetOpportunityListRequest data types
		//	==============================================================
		getOpp_request.setOpportunityID(stringOpportunityID);
		getOpp_request.setCFDANumber(stringCFDANumber);
		getOpp_request.setCompetitionID(stringCompetitionID);
		log.debug("--- Request=" + getOpp_request);
		log.debug("--- OPP ID=" + getOpp_request.getOpportunityID());
		log.debug("--- CFDA=" + getOpp_request.getCFDANumber());
		log.debug("--- COMPID=" + getOpp_request.getCompetitionID());
		//	==============================================================
		//	 Data is sent to the GetOpportunityListProcessor class
		//	==============================================================
		OpportunityInformationType[] getOpp_response = stub.getOpportunityList(getOpp_request);

		//	===============================================================
		//	 Data from _GetOpportunityListListProcess class converted to
		//String data
		//	================================================================

		List l = retrieveOpportunityList(getOpp_response);
		//=================================================================
		// When Records Found or not found
		//=================================================================
		if (l == null) {
			ActionMessages msg = new ActionMessages();
			msg.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage( "messages.none.found"));
			saveMessages(request, msg);
			return mapping.findForward(ACTION_CONTINUE);
		} else {
			request.getSession().setAttribute(GETOPPORTUNITY_LIST, l);
			return mapping.findForward(ACTION_SUCCESS);
		}// if-else
	}

	/**
	 * 
	 * Retrieves opportunities.
	 * 
	 * @param response
	 *            the GetOpportunityListResponse.
	 * @return a list of OpportunityInformationType.
	 * @throws Exception
	 */
	//===================================================
	// Method to add the data to the list
	//===================================================
	public List retrieveOpportunityList(OpportunityInformationType[] oitL)
			throws Exception {

		List al = new ArrayList();

		if (oitL != null) {
		   for (int i = 0; i < oitL.length; i++) {
		      
		      GetOpportunityListInformation GOLI = new GetOpportunityListInformation();
		      
		      GOLI.setOpportunity_id(oitL[i].getOpportunityID());
		      
		      if (oitL[i].getOpportunityTitle() != null) {
		         GOLI.setOpportunity_title(oitL[i].getOpportunityTitle());
		      } else {
		         GOLI.setOpportunity_title("");
		      }
		      
		      if (oitL[i].getOpeningDate() != null) {
		         GOLI.setOpening_date(Utils.formatDate(
		               oitL[i].getOpeningDate().getTime()).substring(
		                     0, 8));
		      } else {
		         GOLI.setOpening_date("");
		      }
		      if (oitL[i].getClosingDate() != null) {
		         GOLI.setClosing_date(Utils.formatDate(
		               oitL[i].getClosingDate().getTime()).substring(
		                     0, 8));
		      } else {
		         GOLI.setClosing_date("");
		      }
		      if (oitL[i].getCFDANumber() != null) {
		         GOLI.setCfda_number(oitL[i].getCFDANumber());
		      } else {
		         GOLI.setCfda_number("");
		      }
		      if (oitL[i].getCompetitionID() != null) {
		         GOLI.setCompetition_id(oitL[i].getCompetitionID());
		      } else {
		         GOLI.setCompetition_id("");
		      }
		      if (oitL[i].getSchemaURL() != null) {
		         GOLI.setSchema_url(oitL[i].getSchemaURL());
		      } else {
		         GOLI.setSchema_url("");
		      }
		      if (oitL[i].getInstructionURL() != null) {
		         GOLI.setInstruction_url(oitL[i].getInstructionURL());
		      } else {
		         GOLI.setInstruction_url("");
		      }
		      
		      al.add(GOLI);
		   }
		}

		return al;
	}

	/**
	 * Validates data
	 * 
	 * @param mapping
	 *            the ActionMapping
	 * @param request
	 *            the HttpServletRequest
	 * @param x
	 *            String
	 * @param y
	 *            String
	 * @param z
	 *            String
	 * @return a boolean
	 */
	private static boolean ValidateData(ActionMapping mapping,
			HttpServletRequest request, String x, String y, String z) {
		if (((request.getParameter(x) == null) || (request.getParameter(x)
				.length() == 0))
				&& ((request.getParameter(y) == null) || (request.getParameter(
						y).length() == 0))
				&& ((request.getParameter(z) == null) || (request.getParameter(
						z).length() == 0))) {

			return false;
		} else {
			return true;
		}

	}
}