/*
 * Created on Jun 15, 2004
 */
package gov.grants.apply.struts.actions;

import gov.grants.apply.WebServices.ApplicantIntegrationServices_V1_0.ApplicantIntegrationSoapBindingStub;
import gov.grants.apply.WebServices.ApplicantIntegrationServices_V1_0.SubmitApplicationRequest;
import gov.grants.apply.WebServices.ApplicantIntegrationServices_V1_0.SubmitApplicationResponse;
import gov.grants.apply.soap.util.SoapUtils;
import gov.grants.apply.struts.Constants;
import gov.grants.apply.struts.forms.SubmitApplicationForm;
import gov.grants.apply.util.Attachment;
import gov.grants.apply.util.GrantApplicationHash;
import gov.grants.apply.util.PerformanceTimer;
import gov.grants.apply.xml.util.InvalidXmlException;
import gov.grants.apply.xml.util.Utils;
import gov.grants.apply.xml.util.Validator;
import gov.grants.apply.xpath.GrantApplicationXpath;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.activation.DataHandler;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.axis.attachments.AttachmentPart;
import org.apache.axis.attachments.DynamicContentDataHandler;
import org.apache.axis.client.Call;
import org.apache.log4j.Logger;
import org.apache.struts.action.ActionError;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;
import org.apache.struts.actions.LookupDispatchAction;
import org.apache.struts.upload.FormFile;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.w3c.dom.Document;
import org.w3c.dom.Node;

/**
 * Action class for uploading and submitting an application with attachments via
 * web services to Grants.gov.
 * 
 * @author Brian Husted
 *  
 */
public class SubmitApplicationAction extends LookupDispatchAction implements
		Constants {

	static final Logger log = Logger.getLogger(SubmitApplicationAction.class
			.getName());

	private static String ENCODING_TYPE = Call.ATTACHMENT_ENCAPSULATION_FORMAT_DIME;
	static {
		try {
			if (SoapUtils.MIME_ENCODING.equalsIgnoreCase(SoapUtils
					.getProperty(SoapUtils.ENCODING_TYPE))) {
				ENCODING_TYPE = Call.ATTACHMENT_ENCAPSULATION_FORMAT_MIME;
				log.debug("SOAP Encoding type set to MIME");
			} else {
				log.debug("SOAP Encoding type set to DIME");
			}
		} catch (Exception ex) {
			log.warn("Unable to determine encoding type.  Defaulting to DIME",
					ex);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.apache.struts.actions.LookupDispatchAction#getKeyMethodMap()
	 */
	protected Map getKeyMethodMap() {
		Map map = new HashMap();
		map.put("button.addAttachment", "addAttachment");
		map.put("button.removeAttachment", "removeAttachments");
		map.put("button.addXML", "addApplicationXml");
		map.put("button.submit", "submitApplication");
		map.put("button.removeXML", "removeApplicationXml");
		map.put("button.validateXML", "validateApplicationXml");
		map.put("button.hashXML", "calculateAndInsertHash");
		return map;
	}

	/**
	 * Submits the application package to the server for processing.
	 * 
	 * @param mapping
	 *            the ActionMapping
	 * @param form
	 *            the ActionForm
	 * @param request
	 *            the HttpServletRequest
	 * @param response
	 *            the HttpServletResponse
	 * @return the ActionForward
	 * @throws Exception
	 */
	public ActionForward submitApplication(ActionMapping mapping,
			ActionForm form, HttpServletRequest request,
			HttpServletResponse response) throws Exception {
		boolean validXml = true;
		String forwardPath = ACTION_SUCCESS;
		try {
			
			String xml = getApplicationXml(request);

			//If it's invalid or null xml then don't call
			//web service.
			if (!isWellFormedXML(request, xml)) {
				forwardPath = ACTION_CONTINUE;
			} else {
				ApplicantIntegrationSoapBindingStub stub = SoapUtils.getApplicantSoapStub();
				//The Grant Application XML is well formed so the application
				// will be submitted
				SubmitApplicationRequest submitRequest = new SubmitApplicationRequest();
				addAttachments( stub, request );
				submitRequest.setGrantApplicationXML(xml);
PerformanceTimer.start();
				SubmitApplicationResponse submitResponse = stub.submitApplication(submitRequest);
PerformanceTimer.stop();
				request.getSession().setAttribute(
								GG_TRACKING_ID,
								submitResponse.getGrants_govTrackingNumber());
				request.getSession().setAttribute(
						APP_RECEIPT_DATE,
						gov.grants.apply.struts.util.Utils
								.formatDate(submitResponse
										.getReceivedDateTime().getTime()));
			}

		} catch (InvalidXmlException ex) {
			log.warn("Invalid XML detected");
			validXml = false;
			forwardPath = ACTION_CONTINUE;
		} finally {

			if (validXml) {
				cleanUp(request);
			}
		}

		return mapping.findForward(forwardPath);

	}

	protected boolean isWellFormedXML(HttpServletRequest request, String xml)
			throws InvalidXmlException {

		boolean xmlValid = true;

		//make sure the xml is in the request
		ActionErrors errors = new ActionErrors();
		if (xml == null) {

			errors.add(ActionErrors.GLOBAL_ERROR, new ActionError(
					"error.applicant.xml.missing"));

			log.warn("Grant Application XML was missiong..");
			xmlValid = false;

		} else if (!Utils.isWellFormedXml(xml)
				|| (xml.indexOf("GrantApplication") < 0)) {
			errors.add(ActionErrors.GLOBAL_ERROR, new ActionError(
					"error.applicant.xml.notXMLFile"));
			log.warn("File was not an XML File");
			xmlValid = false;
		}
		if (!xmlValid) {
			saveErrors(request, errors);
			throw new InvalidXmlException("Invalid XML detected");
		}

		return xmlValid;

	}

	protected void cleanUp(HttpServletRequest request) {
		try {
			deleteApplicationXml(request);
			deleteAttachments(request);

		} catch (Throwable t) {
			log.error("Error during cleanup", t);
		}

	}

	private void deleteAttachments(HttpServletRequest request) {
		Map attachments = (Map) request.getSession().getAttribute(ATTACHMENTS);
		if (attachments == null) {
			//no attachments
			return;
		}

		request.getSession().setAttribute(ATTACHMENTS, null);

	}

	/**
	 * Add the application XML file. Validates that the file is well-formed xml
	 * and contains a GrantApplication element.
	 * 
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return
	 * @throws Exception
	 */
	public ActionForward addApplicationXml(ActionMapping mapping,
			ActionForm form, HttpServletRequest request,
			HttpServletResponse response) throws Exception {
		log.debug("Adding XML");
		try {
			String xml = null;
			FormFile ggXMLFile = ((SubmitApplicationForm) form)
					.getApplicationXML();
			if (ggXMLFile != null && ggXMLFile.getFileName().length() > 0) {
				xml = new String(ggXMLFile.getFileData());
				request.getSession().setAttribute(XML_FILE_NAME,
						ggXMLFile.getFileName());
			}
			if (isWellFormedXML(request, xml)) {
				request.getSession().setAttribute(APPLICATION_XML, xml);
			}
		} catch (InvalidXmlException ex) {
			deleteApplicationXml(request);
		}

		return mapping.findForward(ACTION_CONTINUE);
	}

	/**
	 * Removes the selected xml file from the session.
	 * 
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return
	 * @throws Exception
	 */
	public ActionForward removeApplicationXml(ActionMapping mapping,
			ActionForm form, HttpServletRequest request,
			HttpServletResponse response) throws Exception {
		log.debug("Removing XML");

		deleteApplicationXml(request);
		return mapping.findForward(ACTION_CONTINUE);
	}

	private void deleteApplicationXml(HttpServletRequest request) {
		request.getSession().setAttribute(APPLICATION_XML, null);
		request.getSession().setAttribute(XML_FILE_NAME, null);
	}

	/**
	 * Validates a Grant Application xml file using strict validation.
	 * 
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return
	 * @throws Exception
	 */
	public ActionForward validateApplicationXml(ActionMapping mapping,
			ActionForm form, HttpServletRequest request,
			HttpServletResponse response) throws Exception {
		log.debug("Validating XML");
		String ggXML = getApplicationXml(request);
		Validator avalidator = new Validator();
		try {
			avalidator.validate(new ByteArrayInputStream(ggXML.getBytes()));
			ActionMessages messages = new ActionMessages();
			messages.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(
					"applicant.xml.valid"));
			saveMessages(request, messages);
		} catch (Exception ex) {
			ActionErrors errors = new ActionErrors();
			errors.add(ActionErrors.GLOBAL_ERROR, new ActionError(
					"error.applicant.xml.invalid", ex.getMessage()));
			saveErrors(request, errors);
			log.warn("Grant Application XML was invlid.", ex);
		}

		return mapping.findForward(ACTION_CONTINUE);
	}

	/**
	 * This method calculates the hash value of the
	 * <code>&lt;grant:Forms&gt;</code> tag inside of the uploaded Grant
	 * application XML file. After the computation, the hash value is inserted
	 * into the Header portion of the Grant application XML.
	 * 
	 * @param mapping
	 *            Struts action mapping object.
	 * @param form
	 *            The form bean holding user parameters.
	 * @param request
	 *            The HTTP request object.
	 * @param response
	 *            The HTTP response object.
	 * @return The next action back to the Struts framework.
	 * @throws Exception
	 *             When the application XML cannot be parsed.
	 */
	public ActionForward calculateAndInsertHash(ActionMapping mapping,
			ActionForm form, HttpServletRequest request,
			HttpServletResponse response) throws Exception {
		log.debug("Calculating hash value.");
		String xml = getApplicationXml(request);
		GrantApplicationXpath xpath = new GrantApplicationXpath(xml);
		String hash = GrantApplicationHash.computeGrantFormsHash(xpath);
		if (log.isDebugEnabled()) {
			log.debug("hash value is " + hash);
		}
		ActionMessages messages = new ActionMessages();
		messages.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(
				"applicant.xml.hash", hash));
		saveMessages(request, messages);
		ByteArrayOutputStream stream = new ByteArrayOutputStream();
		XMLSerializer serializer = new XMLSerializer();
		serializer.setOutputByteStream(stream);
		Node hashNode = xpath.getHashNode();
		Document doc = hashNode.getOwnerDocument();
		Node text = hashNode.getFirstChild();
		if (text == null) {
			text = doc.createTextNode(hash);
			hashNode.appendChild(text);
		} else {
			text.setNodeValue(hash);
		}
		OutputFormat format = new OutputFormat(doc);
		format.setPreserveSpace(true);
		serializer.setOutputFormat(format);
		serializer.serialize(doc);
		request.getSession(true).setAttribute(APPLICATION_XML,
				stream.toString());
		log.debug("Inserted hash value into header of XML.");
		return mapping.findForward(ACTION_CONTINUE);
	}

	private String getApplicationXml(HttpServletRequest request)
			throws Exception {
		return (String) request.getSession().getAttribute(APPLICATION_XML);

	}

	private void addAttachments(ApplicantIntegrationSoapBindingStub stub,
			HttpServletRequest request) throws Exception {
		Map attachments = (Map) request.getSession().getAttribute(ATTACHMENTS);
		if (attachments == null || attachments.isEmpty()) {
			//no attachments to process
			return;
		}

		Iterator filesToAttach = attachments.values().iterator();
		while (filesToAttach.hasNext()) {
			Attachment attachment = (Attachment) filesToAttach.next();
			attach(stub, attachment);
		}

	}

	/**
	 * Adds an attachment. Uploads the attachment to the server.
	 * 
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return
	 * @throws Exception
	 */
	public ActionForward addAttachment(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		log.debug("Adding attachment");
		SubmitApplicationForm submitAppform = (SubmitApplicationForm) form;
		FormFile file = submitAppform.getAttachment();
		if (file == null || file.getFileName().trim().length() <= 0) {
			return mapping.findForward(ACTION_CONTINUE);
		}
		log.debug("The file name is: " + file.getFileName());

		Map attachments = (Map) request.getSession().getAttribute(ATTACHMENTS);
		if (attachments == null) {
			attachments = new HashMap();
		}

		if (attachments.containsKey(file.getFileName())) {
			attachments.remove(file.getFileName());
		}

		Attachment attach = gov.grants.apply.struts.util.Utils.createAttachment(file);
		attachments.put(file.getFileName(), attach);

		ActionMessages messages = new ActionMessages();
		messages.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(
				"applicant.attachment.contentid_and_hash", new String[] {
						attach.getAttachmentXML() }));
		log.debug( attach.getAttachmentXML() );
		saveMessages(request, messages);

		request.getSession().setAttribute(ATTACHMENTS, attachments);
		log.debug("Added Attachment: " + file.getFileName());
		return mapping.findForward(ACTION_CONTINUE);
	}

	/**
	 * Removes any attachments that were previously added.
	 * 
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return
	 * @throws Exception
	 */
	public ActionForward removeAttachments(ActionMapping mapping,
			ActionForm form, HttpServletRequest request,
			HttpServletResponse response) throws Exception {
		Map attachments = (Map) request.getSession().getAttribute(ATTACHMENTS);
		String[] filesToDelete = ((SubmitApplicationForm) form)
				.getDeleteAttachments();
		if (attachments != null && filesToDelete != null) {
			for (int i = 0; i < filesToDelete.length; i++) {
				Attachment a = (Attachment) attachments.get(filesToDelete[i]);
				if (a != null) {
					attachments.remove(filesToDelete[i]);
				}
			}
			request.getSession().setAttribute(ATTACHMENTS, attachments);
		}

		return mapping.findForward(ACTION_CONTINUE);
	}

	private void attach(
			ApplicantIntegrationSoapBindingStub stub,
			Attachment attachBean ) 
	throws Exception {

		AttachmentPart attachmentPart = new AttachmentPart();
		DataHandler attachmentFile = new DynamicContentDataHandler(attachBean
				.getByteArrayDS());

		/**
		 * Tell the stub that the message being formed also contains an
		 * attachment, and it is defaulted to DIME encoding.
		 */

		stub._setProperty(Call.ATTACHMENT_ENCAPSULATION_FORMAT, ENCODING_TYPE);
		attachmentPart.setDataHandler(attachmentFile);

		attachmentPart.setContentId(attachBean.getContentId());
		attachmentPart.setContentType(attachBean.getContentType());
		stub.addAttachment(attachmentPart);

	}

}