/**
 *  @filename: StringUtils.java
 *  @create date: 2003-07-14
 *  @author: Grants.gov - Dan Dumitru
 *
 *  @version 1.0.0
 *  @revision date:
 *
 */

package gov.grants.apply.util;

// Sun
import java.util.ArrayList;
import java.util.List;
import java.util.Random;
import java.util.StringTokenizer;
import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;

 // Apache
import org.apache.log4j.Logger;

public final class StringUtils {
    static final Logger log = Logger.getLogger( StringUtils.class.getName() );
	static int scr_key[]={28,1,3,22,26,18,2,15,31,6,24,16,20,9,27,25,19,23,5,13,0,30,4,8,11,29,21,10,14,7,17,12};    

    private StringUtils() {}

    /************************
     *
     *  :: UTILITY METHODS ::
     *
     ************************/

    /**
     *  Replaces all occurences of a the 'oldPattern' in a given
     *  string 'str' with the 'newPattern'
     *
     *  @param str - original string
     *  @param oldPattern - pattern to be replaced
     *  @param newPattern - new pattern to be inserted
     */
    public static String strReplace( String str, String oldPattern, String newPattern ) {
                
        if ( str == null ) { return null; }
        
        int begin = 0;
        int end = 0;
        StringBuffer newString = new StringBuffer();

        while ( ( end = str.indexOf( oldPattern, begin ) ) >= 0 ) {
            newString.append( str.substring( begin, end ) );
            newString.append( newPattern );

            begin = end + oldPattern.length();
        }
        newString.append( str.substring( begin ) );

        return newString.toString();

    }// end strReplace()

    public static String strReplaceIgnoreCase( String str, String oldPattern, String newPattern ) {
        
        if ( str == null ) { return null; }  
        
        int begin = 0;
        int end = 0;
        StringBuffer newString = new StringBuffer();

        String strLowerCase = str.toLowerCase();
        String oldPatternLowerCase = oldPattern.toLowerCase();
        
        while ( ( end = strLowerCase.indexOf( oldPatternLowerCase, begin ) ) >= 0 ) {
            newString.append( str.substring( begin, end ) );
            newString.append( newPattern );

            begin = end + oldPattern.length();
        }
        newString.append( str.substring( begin ) );

        return newString.toString();

    }// end strReplace()

    public static String strTruncate( String str, int begin, int length ) {
        if ( str == null ) { return str; }
        String newValue = null;
        int strLength = str.length();
        int endIndex = begin + length;
        if ( endIndex >= strLength ) { 
            newValue = str.substring( begin );
        } else {
            newValue = str.substring( begin, endIndex ).trim() + "...";
        }// end if-else
        return newValue;
    }// end strTruncate()

    /**
     *  Capitalizes the first letter of the given string 'str'. Returns null if 
     *  null is passed in.  Returns empty string if empty string is passed in
     *
     *  @param str - string to be affected
     */
    public static String initUpperCase( String str ) {
        
        if ( str == null ) { return null; }
        if ( ( str == "" ) || ( str.length() < 1 )) { return str; }
        
        StringBuffer newString = new StringBuffer();

        String firstLetter = str.substring( 0, 1 );
        newString.append( firstLetter.toUpperCase() ).append( str.substring( 1 ) );

        return newString.toString();
    }// initUpperCase()



    public static String setEmptyStrIfNull( String str ) {
        if ( str == null ) {
            return "";
        } else {
            return str;
        }
    }// setEmptyStrIfNull()
  
    public static List stringToList( String str, String delimiter ) {
        if ( str == null || str.trim().length() == 0 ) { return null; }
        
        List list = new ArrayList();
        StringTokenizer token = new StringTokenizer( str, delimiter );
        String value = null;
        while ( token.hasMoreTokens() ) {
            value = token.nextToken();
            list.add( value );
        }
        if ( list.size() == 0 ) { return null; }

        return list;

    }// stringToList()
    
    
    public static String listToString( List list, String delimiter ) {
        
        if ( list == null || list.size() == 0 ) { return null; }
        int listCount = list.size();
        StringBuffer newString = new StringBuffer();
        String value = null;
        for ( int i = 0; i < listCount; i++ ) {
            value = ( String ) list.get( i );
            if ( newString.length() > 0 ) {
                newString.append( delimiter ).append( value );
                
            } else {
                newString.append( value );
                
            }// end if-else
            
        }// end for-loop
        
        return newString.toString();
        
    }// listToString()
    
    
    public static boolean listContainsIgnoreCase( List list, String comparisonString ) {

        if ( list == null || comparisonString == null || comparisonString.trim().length() < 1 ) {
            return false;
        }// if
        boolean containsComparisonString = false;
        String listElement = null;
        for ( int i=0; i < list.size(); i++ ) {
			listElement = ( String )list.get(i);
            if ( listElement.equalsIgnoreCase( comparisonString.trim() ) ) {
                containsComparisonString = true;
                break;
            }// if
        }// for
        return containsComparisonString;
    }// listContainsIgnoreCase()
    
    
    
    public static int listIndexOfIgnoreCase( List list, String comparisonString ) {
        
        int index = -1;
        String listElement = null;
        
        if ( list == null || comparisonString == null || comparisonString.trim().length() < 1 ) {
            return index;
        }

        for ( int i = 0; i < list.size(); i++ ) {
			listElement = ( String )list.get( i );

            if ( listElement.equalsIgnoreCase( comparisonString.trim() ) ) {
                index = i;
                break;
            }
        }

        return index;

    }// listIndexOfIgnoreCase()
    
    
	public static int indexOfIgnoreCase( String str, String fromIndex ) {
        if ( str == null || str.length() == 0 ) { return -1; }
		String lowerStr = str.toLowerCase();
		String lowerFromIndex = fromIndex.toLowerCase();
		return lowerStr.indexOf( lowerFromIndex );
        
	}// indexOfIgnoreCase()
    
    
    /**
     *  Takes a String value, if it is 'null', it will return an
     *  empty string ( "" ), else it will return the String value
     *
     *  @param  str     String value to be evaluated
     *  @return String
     */
    public static String nullToEmptyString( String str ) {
        
        if ( str == null ) { return ""; } 
        
        return str;
        
    }// nullToString()
    
    
    public static String stringArrayToString( String[] strArray ) {
        if ( strArray == null ) { return null; }
        StringBuffer sb = new StringBuffer();
        int count = strArray.length;
        for ( int i = 0; i < count; i++ ) {
            if ( sb.length() == 0 ) {
                sb.append( strArray[ i ] );
            } else {
                sb.append( ", " ).append( strArray[ i ] );
            }// end if-else
            
        }// end for-loop
        
        return sb.toString();
        
    }// stringArrayToString()
    
    /**
     *  This method converts a string array to a float array.
     */    
    public static float[] stringArrayToFloatArray( String[] strArray ) {
    	        
        float[] floatArray = new float[ strArray.length ];
        
        // convert String[] to float[] for hoursWorked

        if ( strArray.length > 0 ) {
            floatArray = new float[ strArray.length ];
            for ( int i = 0; i < strArray.length; i++ ) {
                log.debug( "==> " + i + "=" + strArray[ i ] );
                try {
                    floatArray[ i ] = Float.parseFloat( strArray[ i ] );
                } catch ( NumberFormatException nfe ) {
                    log.debug( "==> INVALID FLOAT [" + strArray[ i ] + "] **" );
                    floatArray[ i ] = 0;
                }
            }// end for-loop
            log.debug( "==> floatArray.length=" + floatArray.length );
        }// end if
        
        return floatArray;
    }    
    
    public static String floatArrayToString( float[] floatArray ) {
        if ( floatArray == null ) { return null; }
        StringBuffer sb = new StringBuffer();
        int count = floatArray.length;
        for ( int i = 0; i < count; i++ ) {
            if ( sb.length() == 0 ) {
                sb.append( floatArray[ i ] );
            } else {
                sb.append( ", " ).append( floatArray[ i ] );
            }// end if-else
            
        }// end for-loop
        
        return sb.toString();
        
    }// floatArrayToString()
    
    public static String makeSqlReady(String str) {
        
        if (str == null) return null;
        
        String ret="";
        StringTokenizer st = new StringTokenizer(str,"'",true);

        while(st.hasMoreTokens()) {
            String t_str = st.nextToken();
            if (t_str.charAt(0)=='\'') ret+=(t_str+t_str);
            else ret+=t_str;
        }
        
        return ret;  
    } // makeSqlReady()
    
    public static String makeSqlReadyHandleNull( String str ) {
        
        String newString = makeSqlReady( str );
        
        if ( newString != null ) {
            return "'" + newString + "'";
        }
        // else return null
        return newString;
        
    } // makeSqlReady() 
    
	public static String encrypt( int cust_id ) {
		long key		= 691278383;
		long data[]		= new long[2];
		long my_ret[]   = new long[4];
		data[0]			= System.currentTimeMillis()/1000;
		data[1]			= cust_id;
		
		byte my_seed[]	= new byte[4];
		Random new_gen	= new Random();
		new_gen.nextBytes(my_seed);
		long seed		= my_ret[0] = (long) (my_seed[0]+128)<<24 | (my_seed[1]+128)<<16 | (my_seed[2]+128)<<8 | (my_seed[3]+128);

		for (int i=0;i<2;i++) {
			seed=scrumble(seed^data[i]);
			my_ret[i+1]=(long) seed^key;
		}
		String out_str = "";
		for (int i=0;i<3;i++) {
			String t = Long.toHexString(my_ret[i]).toUpperCase();
			while (t.length()<8) t="0"+t;
			out_str+=t;
		}
		return out_str;
	}   
    
	private static long scrumble( long n ) {
		long rv=0;
		for (int i=0;i<32;i++) {
			rv|=(long) (n>>i & 1)<<scr_key[i];
		}
		return rv;
	}    

    public static String replaceQuestionMark( String baseString, String insertString ) {
		int				markLoc			= 0;
		String			returnString	= null;

		// set the correct table names into the statement	
		markLoc = baseString.indexOf( '?' );
		if ( markLoc == -1 )  { return null; }
		returnString = baseString.substring( 0, markLoc ) + insertString + baseString.substring( markLoc + 1, baseString.length() );		

        return returnString;
    }// replaceQuestionMark()
    
    public static String extractDBval( String value ) {
        return value != null ? value : "";
    } // extractDBVal()
	
	// Escape Quotes in strings for use in drop downs, etc..
	public static String escapeQuotes(String str)
	{
		if ( str == null ) {
			return "";
		}

		int pos = 0;
		StringBuffer strBuff = new StringBuffer(str);
		
		while(pos < strBuff.length())
		{
			if(strBuff.charAt(pos) == '\'')			// escape ' chars
			{
				strBuff.replace(pos, pos+1, "\\'");
				pos++;
			}
			else if(strBuff.charAt(pos) == '"') 	// escape " chars
			{
				strBuff.replace(pos, pos+1, "\\\"");
				pos++;
			}
			pos++;
		}
		
		return strBuff.toString();
	}	
    
    /**
     *  Servers as a 'toString()' method for objects that contain
     *  'getter' methods.
     *
     *  @param      obj
     *  @return     String
     *  @exception  Exception
     */
    public static String getToString( Object obj ) {
        StringBuffer toString = new StringBuffer();
		String methodName = null;
        try {
            Class myClass = obj.getClass();

            Method[] methods = myClass.getDeclaredMethods();
            int count = methods.length;

            Method method = null;
            methodName = null;
            Object value = null;

            for ( int i = 0; i < count; i++ ) {
                method = methods[ i ];
                methodName = method.getName();

                // use only 'getter' methods with no parameters
                if ( methodName.substring( 0, 3 ).equals( "get" ) && method.getParameterTypes().length == 0 ) {
//					log.debug( "==> methodName=" + methodName );
//                  returnType = method.getReturnType();
					try {
						value = method.invoke( obj, null );
					} catch ( InvocationTargetException e ) {
						toString.append( "I'm sorry  :-(  I cannot perform toString() for " + obj.getClass().getName() + " object, method name: " + methodName );
					}
//                  log.debug( "==> value=" + value );
                    toString.append( System.getProperty( "line.separator" ) ).append( "[" ).append( methodName ).append( "()=" ).append( value ).append( "]" );
                }// if
            }// for
        } catch ( Exception e ) {
            log.debug( "ERROR :: Exception thrown:\n" + e + " :: cause ::  " + e.getMessage() );
//            toString.setLength( 0 );
            toString.append( "I'm sorry  :-(  I cannot perform toString() for " + obj.getClass().getName() + " object, method name: " + methodName );
        }
        return toString.toString();
    }// getToString()

    /**
     *  Method works only for String values
     */
    public static Object getObjectFromList( String fieldName, String value, List objList ) {
        if ( objList == null || objList.size() == 0 || 
             fieldName == null || fieldName.length() == 0 ) { return null; }
        int count = objList.size();
        String requestedMethod = "get" + fieldName;
        Object obj = null;
        Class myClass = null;
        Method[] methods = null;
        Method method = null;
        String methodName = null;
        int methodCount = 0;
        Object methodValue = null;
        
        try {
            for (int i = 0; i < count; i++ ) {
                obj = (Object)objList.get( i );
//                log.debug("##### obj = " + obj );
//                log.debug("##### obj = " + getToString( obj ) );
                myClass = obj.getClass();
                methods = myClass.getDeclaredMethods();
                methodCount = methods.length;
//                log.debug("##### methodCount = " + methodCount );
                for ( int j = 0; j < methodCount; j++ ) {
                    method = methods[j];
                    methodName = method.getName();
//                    log.debug( "==> methodName=" + methodName );
                    // use only 'getter' methods with no parameters
                    if ( methodName.equalsIgnoreCase( requestedMethod ) && method.getParameterTypes().length == 0 ) {
//    					log.debug( "==> methodName=" + methodName );
//                        log.debug( "==> method=" + method );
                        methodValue = method.invoke( obj, null );
//                        log.debug( "==> methodValue=" + String.valueOf( methodValue ) );
                        if ( value.equalsIgnoreCase( String.valueOf( methodValue ) ) ) {
                            return obj;
                        }
                    }// if
                    
                }
                
            } 
        } catch ( Exception e ) {
            log.debug( "ERROR :: Exception thrown:\n" + e + " :: cause ::  " + e.getMessage() );
        }// try-catch
        // return null ... since object now
        return null;
    }// getObjectFromList()
    
    /**
     * <p>Return <code>true</code> if the specified object is a String or
     * a <code>null</code> value.</p>
     *
     * @param obj Object to be tested
     */
    public static boolean isString( Object obj ) {
        if ( obj == null ) { return ( true ); }
        return ( String.class.isInstance( obj ) );
    }// isString()
    
    
    /**********************************
     *
     *  :: STRING VALIDATION METHODS ::
     *
     **********************************/
    
    public static boolean isAlphaNumeric( String str ) {
        if ( str == null || str.trim().length() < 1 ) { return false; }
        
        char[] charArray = str.toCharArray();
        int count = charArray.length;
        char ch;
        for ( int i = 0; i < count; i++ ) {
            ch = charArray[ i ];
            // exit if invalid
            if ( !Character.isLetterOrDigit( ch ) ) { return false; }
        }// for
        
        return true;// valid
    }// isAlphaNumeric()

    public static boolean isAlpha( String str ) {
        if ( str == null || str.trim().length() < 1 ) { return false; }
        
        char[] charArray = str.toCharArray();
        int count = charArray.length;
        char ch;
        for ( int i = 0; i < count; i++ ) {
            ch = charArray[ i ];
            // exit if invalid
            if ( !Character.isLetter( ch ) ) { return false; }
        }// for
        
        return true;// valid
    }// isAlphaNumeric()

}// StringUtils