/*
 * Created on Jun 1, 2005
 * Unpublished Work, Copyright, 2005 Northrop Grumman
 */
package gov.grants.apply.util;

import java.io.InputStream;

import javax.servlet.ServletContext;

import org.apache.log4j.Logger;

/**
 * @author Grants.gov WebServices Team
 *
 * This is a wrapper class for the ServletContext object.
 */
public final class WebAppContext
{
   private static final Logger LOG = Logger.getLogger(WebAppContext.class);
   private ServletContext _context = null;
   private static WebAppContext _instance = null;
   
   void setServletContext(ServletContext context)
   {
      _context = context;
   }
   
   private WebAppContext()
   {
   }
   
   /**
    * @return The singleton instance.
    */
   public static WebAppContext getInstance()
   {
      if (_instance == null)
      {
         _instance = new WebAppContext();
      }
      return _instance;
   }
   
   /**
    * Returns the resource at the given path.
    * @param path Location of resource relative to the web application root.
    * @return The resource as an InputStream or null if resource is not found.
    */
   public InputStream getResourceAsStream(String path)
   {
      return getServletContext().getResourceAsStream(path);
   }
   
   /**
    * Returns the full absolute path as seen on the computer.
    * @param virtualPath A path that is relative to the web application root.
    * @return The absolute path of the location given by the virtualPath argument or null if not found.
    */
   public String getRealPath(String virtualPath)
   {
      return getServletContext().getRealPath(virtualPath);
   }
   
   /**
    * @return The ServletContext of the web application that is running your Axis server.
    */
   public ServletContext getServletContext()
   {
      if (_context == null)
      {
         _context = AxisUtils.getCurrentHttpRequest().getSession(true).getServletContext();
      }
      if (_context == null)
      {
         throw new IllegalStateException("WebAppContext has not been initialized, ServletContext is null and could not be found under the current Axis' MessageContext.");
      }
      return _context;
   }
}
